@@ -209,11 +209,12 @@ unsigned IsJava (unsigned EvType)
 /******************************************************************************
  ***  IsCUDA
  ******************************************************************************/
-#define CUDA_EVENTS 8
+#define CUDA_EVENTS 9
 static unsigned cuda_events[] = {
 	/* Host events */
 	CUDACALL_EV, CUDACALLGPU_EV, CUDA_DYNAMIC_MEM_PTR_EV, CUDA_DYNAMIC_MEM_SIZE_EV,
-	CUDA_KERNEL_INST_EV, CUDASTREAMBARRIER_THID_EV, CUDA_KERNEL_EXEC_EV, CUDA_UNTRACKED_EV
+	CUDA_KERNEL_INST_EV, CUDA_STREAM_DEST_ID_EV, CUDA_KERNEL_EXEC_EV, CUDAEVENT_ID_EV, 
+	CUDA_UNTRACKED_EV
 };
 
 unsigned IsCUDA (unsigned EvType)
@@ -226,6 +227,26 @@ unsigned IsCUDA (unsigned EvType)
   return FALSE;
 }
 
+/******************************************************************************
+ ***  IsHIP
+ ******************************************************************************/
+#define HIP_EVENTS 8
+static unsigned hip_events[] = {
+	/* Host events */
+	HIPCALL_EV, HIPCALLGPU_EV, HIP_DYNAMIC_MEM_PTR_EV, HIP_DYNAMIC_MEM_SIZE_EV,
+	HIP_KERNEL_INST_EV, HIPSTREAMBARRIER_THID_EV, HIP_KERNEL_EXEC_EV, HIP_UNTRACKED_EV
+};
+
+unsigned IsHIP (unsigned EvType)
+{
+  unsigned evt;
+
+  for (evt = 0; evt < HIP_EVENTS; evt++)
+    if (hip_events[evt] == EvType)
+      return TRUE;
+  return FALSE;
+}
+
 /******************************************************************************
  ***  IsOpenCL
  ******************************************************************************/
@@ -474,6 +495,11 @@ EventType_t getEventType (unsigned EvType, unsigned *Type)
 		*Type = CUDA_TYPE;
 		return TRUE;
 	}
+	else if (IsHIP(EvType))
+	{
+		*Type = HIP_TYPE;
+		return TRUE;
+	}
 	else if (IsOpenCL(EvType))
 	{
 		*Type = OPENCL_TYPE;
