@@ -562,7 +562,7 @@ enum {
 	CUDACONFIGCALL_VAL,
 	CUDAMEMCPY_VAL,
 	CUDATHREADBARRIER_VAL,
-	CUDASTREAMBARRIER_VAL,
+	CUDASTREAMSYNCHRONIZE_VAL,
 	CUDASTREAMCREATE_VAL,
 	CUDAMEMCPYASYNC_VAL,
 	CUDADEVICERESET_VAL,
@@ -582,8 +582,8 @@ enum {
 	CUDAMEMCPYFROMARRAY_VAL,        /*TBD*/
 	CUDAMEMCPY2DFROMARRAY_VAL,      /*TBD*/
 	CUDAMEMCPYARRAYTOARRAY_VAL,     /*TBD*/
-	CUDAMEMCPYTOSYMBOL_VAL,         /*TBD*/
-	CUDAMEMCPYFROMSYMBOL_VAL,       /*TBD*/
+	CUDAMEMCPYTOSYMBOL_VAL,
+	CUDAMEMCPYFROMSYMBOL_VAL,
 	CUDAMEMCPYTOARRAYASYNC_VAL,     /*TBD*/
 	CUDAMEMCPYFROMARRAYASYNC_VAL,   /*TBD*/
 	CUDAMEMCPY2DASYNC_VAL,          /*TBD*/
@@ -592,31 +592,100 @@ enum {
 	CUDAMEMCPYTOSYMBOLASYNC_VAL,    /*TBD*/
 	CUDAMEMCPYFROMSYMBOLASYNC_VAL,  /*TBD*/
 	CUDAMEMSET_VAL,
+	CUDAMEMSETASYNC_VAL,
 	CUDAEVENTRECORD_VAL,
-	CUDAEVENTSYNCHRONIZE_VAL
+	CUDAEVENTSYNCHRONIZE_VAL,
+	CUDASTREAMWAITEVENT_VAL,
+	CUDAMALLOC3D_VAL,
+	CUDAMALLOC3DARRAY_VAL,
 };
 
-#define CUDACALLGPU_EV       63000001
+#define CUDACALLGPU_EV       63100000
 enum {
-	CUDAKERNEL_GPU_VAL        = 1,
+	CUDAKERNEL_GPU_VAL           = 1,
 	CUDACONFIGKERNEL_GPU_VAL,
 	CUDAMEMCPY_GPU_VAL,
-	CUDAMEMCPYASYNC_GPU_VAL   = 7,
-	CUDAEVENTRECORD_GPU_VAL = 35
+	CUDAMEMCPYASYNC_GPU_VAL      = CUDAMEMCPYASYNC_VAL,
+	CUDAMEMCPYTOSYMBOL_GPU_VAL   = CUDAMEMCPYTOSYMBOL_VAL,
+	CUDAMEMCPYFROMSYMBOL_GPU_VAL = CUDAMEMCPYFROMSYMBOL_VAL
 };
 
+#define CUDA_MEMORY_TRANSFER          63000001
 #define CUDA_UNTRACKED_EV             63000003
 #define CUDA_DYNAMIC_MEM_SIZE_EV      63000004
 #define CUDA_DYNAMIC_MEM_PTR_EV       63000005
-#define CUDA_KERNEL_INST_EV           63000006
-#define CUDA_KERNEL_INST_LINE_EV      63000007
-#define CUDASTREAMBARRIER_THID_EV     63000008  /* TID of the Paraver thid of the CUDA stream being synchronized */
-#define CUDA_KERNEL_EXEC_EV           63000009
-#define CUDA_KERNEL_EXEC_LINE_EV      63000010
+#define CUDA_KERNEL_EXEC_EV           63000006
+#define CUDA_KERNEL_EXEC_LINE_EV      63000007
+#define CUDA_STREAM_DEST_ID_EV        63000008  // Stream ID(Paraver TID) in which CUDA events are executed (target stream for record/sync operations).
+#define CUDA_KERNEL_INST_EV           63000009
+#define CUDA_KERNEL_INST_LINE_EV      63000010
 #define CUDA_KERNEL_BLOCKS_PER_GRID   63000011
 #define CUDA_KERNEL_THREADS_PER_BLOCK 63000012
 #define CUDA_KERNEL_SHARED_MEM_BYTES  63000013
-#define CUDA_MEMORY_TRANSFER          63000014
+#define CUDAEVENT_ID_EV               63000014
+
+
+#define HIPCALL_EV               63500000
+enum {
+	HIPLAUNCH_VAL                 =     1,
+	HIPCONFIGCALL_VAL,
+	HIPMEMCPY_VAL,
+	HIPTHREADBARRIER_VAL,
+	HIPSTREAMBARRIER_VAL,
+	HIPSTREAMCREATE_VAL,
+	HIPMEMCPYASYNC_VAL,
+	HIPDEVICERESET_VAL,
+	HIPTHREADEXIT_VAL,
+	HIPSTREAMDESTROY_VAL,
+	HIPMALLOC_VAL,
+	HIPMALLOCPITCH_VAL,
+	HIPFREE_VAL,
+	HIPMALLOCARRAY_VAL,
+	HIPFREEARRAY_VAL,
+	HIPMALLOCHOST_VAL,
+	HIPFREEHOST_VAL,
+	HIPHOSTALLOC_VAL,
+	HIPMEMCPY2D_VAL,               /*TBD*/
+	HIPMEMCPYTOARRAY_VAL,          /*TBD*/
+	HIPMEMCPY2DTOARRAY_VAL,        /*TBD*/
+	HIPMEMCPYFROMARRAY_VAL,        /*TBD*/
+	HIPMEMCPY2DFROMARRAY_VAL,      /*TBD*/
+	HIPMEMCPYARRAYTOARRAY_VAL,     /*TBD*/
+	HIPMEMCPYTOSYMBOL_VAL,         /*TBD*/
+	HIPMEMCPYFROMSYMBOL_VAL,       /*TBD*/
+	HIPMEMCPYTOARRAYASYNC_VAL,     /*TBD*/
+	HIPMEMCPYFROMARRAYASYNC_VAL,   /*TBD*/
+	HIPMEMCPY2DASYNC_VAL,          /*TBD*/
+	HIPMEMCPY2DTOARRAYASYNC_VAL,   /*TBD*/
+	HIPMEMCPY2DFROMARRAYASYNC_VAL, /*TBD*/
+	HIPMEMCPYTOSYMBOLASYNC_VAL,    /*TBD*/
+	HIPMEMCPYFROMSYMBOLASYNC_VAL,  /*TBD*/
+	HIPMEMSET_VAL,
+	HIPEVENTRECORD_VAL,
+	HIPEVENTSYNCHRONIZE_VAL
+};
+
+#define HIPCALLGPU_EV       63500001
+enum {
+	HIPKERNEL_GPU_VAL        = 1,
+	HIPCONFIGKERNEL_GPU_VAL,
+	HIPMEMCPY_GPU_VAL,
+	HIPMEMCPYASYNC_GPU_VAL   = 7,
+	HIPEVENTRECORD_GPU_VAL = 35
+};
+
+#define HIP_UNTRACKED_EV             63500003
+#define HIP_DYNAMIC_MEM_SIZE_EV      63500004
+#define HIP_DYNAMIC_MEM_PTR_EV       63500005
+#define HIP_KERNEL_INST_EV           63500006
+#define HIP_KERNEL_INST_LINE_EV      63500007
+#define HIPSTREAMBARRIER_THID_EV     63500008  /* TID of the Paraver thid of the HIP stream being synchronized */
+#define HIP_KERNEL_EXEC_EV           63500009
+#define HIP_KERNEL_EXEC_LINE_EV      63500010
+#define HIP_KERNEL_BLOCKS_PER_GRID   63500011
+#define HIP_KERNEL_THREADS_PER_BLOCK 63500012
+#define HIP_KERNEL_SHARED_MEM_BYTES  63500013
+#define HIP_MEMORY_TRANSFER          63500014
 
 #define OPENCL_KERNEL_NAME_EV                         64200000
 #define OPENCL_BASE_TYPE_EV                           64000000
@@ -1113,6 +1182,7 @@ typedef enum
 	OPENMP_TYPE,
 	PTHREAD_TYPE,
 	CUDA_TYPE,
+	HIP_TYPE,
 	OPENCL_TYPE,
 	OPENSHMEM_TYPE,
 	JAVA_TYPE,
